/* global BroadcastChannel */
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.injectContext = injectContext;
exports.resetInstanceContext = resetInstanceContext;

var _builtin = require('./builtin');

var _emitter = require('./emitter');

var _emitter2 = _interopRequireDefault(_emitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NativeComponents = {};
var NativeModules = {};

var Document = void 0;
var Element = void 0;
var Comment = void 0;

var MODULE_NAME_PREFIX = '@weex-module/';
var MODAL_MODULE = MODULE_NAME_PREFIX + 'modal';
var NAVIGATOR_MODULE = MODULE_NAME_PREFIX + 'navigator';
var GLOBAL_EVENT_MODULE = MODULE_NAME_PREFIX + 'globalEvent';
var noop = function noop() {};
var weex = {};

function genBuiltinModules(modules, moduleFactories, context) {
  for (var moduleName in moduleFactories) {
    modules[moduleName] = {
      factory: moduleFactories[moduleName].bind(context),
      module: { exports: {} },
      isInitialized: false
    };
  }
  return modules;
}

function injectContext() {
  var instanceContext = new Function('return this')();
  var window = resetInstanceContext(instanceContext);
  for (var key in window) {
    if (typeof instanceContext[key] === 'undefined' && key != '__weex_data__') {
      instanceContext[key] = window[key];
    }
  }
}

/**
 * create a Weex instance
 *
 * @param  {string} instanceId
 * @param  {string} __weex_code__
 * @param  {object} [__weex_options__] {bundleUrl, debug}
 */
function resetInstanceContext(instanceContext) {
  var instanceId = instanceContext.instanceId,
      document = instanceContext.document,
      bundleUrl = instanceContext.bundleUrl,
      __weex_document__ = instanceContext.__weex_document__,
      __weex_options__ = instanceContext.__weex_options__,
      __weex_data__ = instanceContext.__weex_data__,
      __weex_config__ = instanceContext.__weex_config__;


  weex = __weex_options__.weex;

  // Mark start time
  var responseEnd = Date.now();
  var __weex_env__ = (typeof WXEnvironment === 'undefined' ? 'undefined' : _typeof(WXEnvironment)) === 'object' && WXEnvironment || {};
  // For better performance use built-in promise first
  var shared = require('runtime-shared/dist/shared.function')();

  var Promise = typeof Promise === 'function' ? Promise : shared.Promise;
  var _Symbol = typeof _Symbol === 'function' ? _Symbol : shared.Symbol;
  var Set = typeof Set === 'function' ? Set : shared.Set;
  var Map = typeof Map === 'function' ? Map : shared.Map;
  var WeakMap = typeof WeakMap === 'function' ? WeakMap : shared.WeakMap;
  var WeakSet = typeof WeakSet === 'function' ? WeakSet : shared.WeakSet;
  var URL = shared.URL,
      URLSearchParams = shared.URLSearchParams,
      FontFace = shared.FontFace,
      matchMedia = shared.matchMedia;


  var documentURL = new URL(bundleUrl);
  var modules = {};

  // Generate native modules map at instance init
  var __weex_define__ = require('./define.weex')(modules);
  var __weex_require__ = require('./require.weex')(modules, weex);
  var __weex_downgrade__ = require('./downgrade.weex')(__weex_require__);
  // Extend document
  require('./document.weex')(__weex_require__, document);

  var location = require('./location.weex')(__weex_require__, documentURL);

  var _require = require('./fetch.weex')(__weex_require__, Promise),
      fetch = _require.fetch,
      Headers = _require.Headers,
      Request = _require.Request,
      Response = _require.Response;

  var XMLHttpRequest = require('./xmlhttprequest.weex')(__weex_require__);
  var WebSocket = require('./websocket.weex')(__weex_require__);

  var _require2 = require('./timer.weex')(__weex_require__, document),
      setTimeout = _require2.setTimeout,
      clearTimeout = _require2.clearTimeout,
      setInterval = _require2.setInterval,
      clearInterval = _require2.clearInterval,
      requestAnimationFrame = _require2.requestAnimationFrame,
      cancelAnimationFrame = _require2.cancelAnimationFrame;

  var _require3 = require('./base64.weex')(),
      atob = _require3.atob,
      btoa = _require3.btoa;

  var performance = require('./performance.weex')(responseEnd);

  var _require4 = require('./event.weex')(),
      Event = _require4.Event,
      CustomEvent = _require4.CustomEvent;

  var windowEmitter = new _emitter2.default();

  var errorHandler = null;
  function registerErrorHandler() {
    if (registerErrorHandler.once) return;

    var globalEvent = __weex_require__(GLOBAL_EVENT_MODULE);
    globalEvent.addEventListener('exception', function (e) {
      // TODO: miss lineno and colno
      // window.onerror = function(messageOrEvent, source, lineno, colno, error) { ... }
      errorHandler(e.exception, e.bundleUrl, 0, 0, new Error(e.exception, e.bundleUrl, 0));
    });

    registerErrorHandler.once = true;
  }

  var window = {
    // ES
    Promise: Promise,
    Symbol: _Symbol,
    Map: Map,
    Set: Set,
    WeakMap: WeakMap,
    WeakSet: WeakSet,
    // W3C: https://www.w3.org/TR/html5/browsers.html#browsing-context-name
    name: '',
    // This read-only property indicates whether the referenced window is closed or not.
    closed: false,
    atob: atob,
    btoa: btoa,
    performance: performance,
    // W3C
    document: document,
    location: location,
    // https://www.w3.org/TR/2009/WD-html5-20090423/browsers.html#dom-navigator
    navigator: {
      product: 'Weex',
      platform: __weex_env__.platform,
      appName: __weex_env__.appName,
      appVersion: __weex_env__.appVersion,
      // Weex/0.12 iOS/9.3 (iPhone7,2) AppName/0.12
      userAgent: 'Weex/' + __weex_env__.weexVersion + ' ' + __weex_env__.platform + '/' + __weex_env__.osVersion + ' (' + __weex_env__.deviceModel + ') ' + __weex_env__.appName + '/' + __weex_env__.appVersion
    },
    // https://drafts.csswg.org/cssom-view/#the-screen-interface
    screen: {
      width: __weex_env__.deviceWidth,
      height: __weex_env__.deviceHeight,
      availWidth: __weex_env__.deviceWidth,
      availHeight: __weex_env__.deviceHeight,
      colorDepth: 24,
      pixelDepth: 24
    },
    devicePixelRatio: __weex_env__.scale,
    fetch: fetch,
    Headers: Headers,
    Response: Response,
    Request: Request,
    XMLHttpRequest: XMLHttpRequest,
    URL: URL,
    URLSearchParams: URLSearchParams,
    FontFace: FontFace,
    WebSocket: WebSocket,
    Event: Event,
    CustomEvent: CustomEvent,
    matchMedia: matchMedia,
    setTimeout: setTimeout,
    clearTimeout: clearTimeout,
    setInterval: setInterval,
    clearInterval: clearInterval,
    requestAnimationFrame: requestAnimationFrame,
    cancelAnimationFrame: cancelAnimationFrame,
    setImmediate: typeof setImmediate === 'function' && setImmediate || function (fn) {
      setTimeout(fn, 0);
    },
    clearImmediate: typeof setImmediate === 'function' && setImmediate || function (id) {
      clearTimeout(id);
    },
    frameworkVersion: '0.6.0', // for debug
    alert: function alert(message) {
      var modal = __weex_require__(MODAL_MODULE);
      modal.alert({
        message: message
      }, function () {});
    },
    open: function open(url) {
      var weexNavigator = __weex_require__(NAVIGATOR_MODULE);
      weexNavigator.push({
        url: url,
        animated: true
      }, noop);
    },
    close: function close() {
      var weexNavigator = __weex_require__(NAVIGATOR_MODULE);
      weexNavigator.close({
        animated: true
      }, noop, noop);
    },
    postMessage: function postMessage(message, targetOrigin) {
      var event = {
        origin: location.origin,
        data: JSON.parse(JSON.stringify(message)),
        type: 'message',
        source: window // FIXME: maybe not export window
      };

      if (typeof BroadcastChannel === 'function') {
        if (targetOrigin == '*') {
          var stack = new BroadcastChannel('message');
          stack.postMessage(event);
        } else {
          var stack = new BroadcastChannel('message' + targetOrigin);
          stack.postMessage(event);
        }
      }
    },
    addEventListener: function addEventListener(type, listener) {
      if (type === 'message') {
        if (typeof BroadcastChannel === 'function') {
          var stack = new BroadcastChannel('message');
          var thisStack = new BroadcastChannel('message' + bundleUrl);
          stack.onmessage = function (e) {
            listener(e.data);
          };
          thisStack.onmessage = function (e) {
            listener(e.data);
          };
        }
      } else {
        windowEmitter.on(type, listener);
      }
    },
    removeEventListener: function removeEventListener(type, listener) {
      windowEmitter.off(type, listener);
    },
    dispatchEvent: function dispatchEvent(e) {
      windowEmitter.emit(e.type, e);
    },
    set onerror(handler) {
      if (typeof handler == 'function') {
        errorHandler = handler;
        registerErrorHandler();
      } else {
        errorHandler = null;
      }
    },
    get onerror() {
      return errorHandler;
    },
    // ModuleJS
    define: __weex_define__,
    require: __weex_require__,
    // Weex
    callNative: function callNative() {}, // compatible original API
    __weex_document__: __weex_document__,
    __weex_module_supports__: weex.isRegisteredModule,
    __weex_tag_supports__: weex.isRegisteredComponent,
    __weex_define__: __weex_define__,
    __weex_require__: __weex_require__,
    __weex_downgrade__: __weex_downgrade__,
    __weex_env__: __weex_env__,
    __weex_code__: '',
    __weex_options__: __weex_options__,
    __weex_data__: __weex_data__,
    __weex_config__: __weex_config__
  };

  genBuiltinModules(modules, _builtin.ModuleFactories, window);

  window.self = window.window = window;

  return window;
}
exports['default'] = module.exports;
exports.default = module.exports;